/*
 *  BCCPM.cpp
 *  
 *  Created by Zhichao Lian on Apr/10/2013.
 *  Copyright 2013 Yale University. All rights reserved.
 *
 */

#include "BCCPM.h"

BCCPM::BCCPM()
{
	num_sample = 0;
	num_pos = 0;
	
	num_iter_1 = 0;
	
	change_point_prior = -1;
	
	data = NULL;
}

BCCPM::~BCCPM()
{
	if( NULL != data )
	{
		gsl_matrix_free( data );
	}
}

int BCCPM::propose_divide( double & new_post, vector< BCCPMBlock > & new_blocks, gsl_rng * rng )
{
	long block_num = gsl_rng_uniform_int( rng, new_blocks.size() );

	long block_start = new_blocks[block_num].get_start();
	long block_num_sample = new_blocks[block_num].get_num_sample();
	
	if( block_num_sample <= 1 )
	{
		new_post = GSL_NEGINF;
		return 0;
	}
	long change_num = gsl_rng_uniform_int( rng, block_num_sample - 1 );
	change_num ++;

	BCCPMBlock block_1( num_pos, hyper_par_group, hyper_par_single );
	BCCPMBlock block_2( num_pos, hyper_par_group, hyper_par_single );
	
	block_1.set_data( block_start, change_num, data );
	block_2.set_data( block_start + change_num, block_num_sample - change_num, data );
	
	long num_sample_1 = block_1.get_num_sample();
	long num_sample_2 = block_2.get_num_sample();
	
	if( num_sample_1 < 2 || num_sample_2 < 2 )
	{
		new_post = GSL_NEGINF;
		return 0;	
	}
	block_1.calc_post();
	block_2.calc_post();
//	block_1.initialize();
//	block_2.initialize();
	
//	block_1.metro_hast_run( num_iter_2, 10 );
//	block_2.metro_hast_run( num_iter_2, 10 );
	
	vector< BCCPMBlock > temp_blocks;
	temp_blocks.clear();
	new_post = 0;
	
	for( long i = 0; i < new_blocks.size(); i ++ )
	{
		if( i == block_num )
		{
			temp_blocks.push_back( block_1 );
			temp_blocks.push_back( block_2 );
			new_post += block_1.get_current_post() + change_point_prior;
			new_post += block_2.get_current_post() + change_point_prior;
		}else
		{
			temp_blocks.push_back( new_blocks[i] );
			new_post += new_blocks[i].get_current_post() + change_point_prior;
		}
	}
    
	new_blocks = temp_blocks;
	return 0;
}

int BCCPM::propose_combine( double & new_post, vector< BCCPMBlock > & new_blocks, gsl_rng * rng )
{
	if( new_blocks.size() <= 1 )
	{
		new_post = GSL_NEGINF;
		return 0;
	}
	
	long block_num = gsl_rng_uniform_int( rng, new_blocks.size() - 1 );
	long block_start = new_blocks[block_num].get_start();
	long block_num_sample = new_blocks[block_num].get_num_sample() + new_blocks[block_num+1].get_num_sample();
	BCCPMBlock combined_block( num_pos, hyper_par_group, hyper_par_single );
	combined_block.set_data( block_start, block_num_sample, data );
	combined_block.calc_post();

//	combined_block.initialize();
//	combined_block.metro_hast_run( num_iter_2, 10 );
	
	vector< BCCPMBlock > temp_blocks;
	temp_blocks.clear();
	new_post = 0;
	for( long i = 0; i < new_blocks.size(); i ++ )
	{
		if( i == block_num )
		{
			temp_blocks.push_back( combined_block );
			new_post += combined_block.get_current_post() + change_point_prior;
			i ++;
		}else
		{
			temp_blocks.push_back( new_blocks[i] );
			new_post += new_blocks[i].get_current_post() + change_point_prior;
		}
	}
	new_blocks = temp_blocks;
	return 0;
}



int BCCPM::propose_shift( double & new_post, vector< BCCPMBlock > & new_blocks, gsl_rng * rng )
{
	if( new_blocks.size() <= 1 )
	{
		new_post = GSL_NEGINF;
		return 0;
	}
	
	long block_num = gsl_rng_uniform_int( rng, new_blocks.size() - 1 );
	
	BCCPMBlock block_1( num_pos, hyper_par_group, hyper_par_single );
	BCCPMBlock block_2( num_pos, hyper_par_group, hyper_par_single );
	
	long block_start_1 = new_blocks[block_num].get_start();
	long block_start_2 = new_blocks[block_num+1].get_start();
	long num_sample_1 = new_blocks[block_num].get_num_sample();
	long num_sample_2 = new_blocks[block_num+1].get_num_sample();
	
	int right = gsl_rng_uniform_int( rng, 2 );
	
	if( right )
	{
		block_start_2 += 1;
		num_sample_1 += 1;
		num_sample_2 -= 1;
	}else
	{
		block_start_2 -= 1;
		num_sample_1 -= 1;
		num_sample_2 += 1;
	}
	
	if( num_sample_1 < 2 || num_sample_2 < 2 )
	{
		new_post = GSL_NEGINF;
		return 0;	
	}
	
	block_1.set_data( block_start_1, num_sample_1, data );
	block_2.set_data( block_start_2, num_sample_2, data );
	
	block_1.calc_post();
	block_2.calc_post();

//	block_1.initialize();
//	block_2.initialize();
	
//	block_1.metro_hast_run( num_iter_2, 10 );
//	block_2.metro_hast_run( num_iter_2, 10 );
	
	vector< BCCPMBlock > temp_blocks;
	temp_blocks.clear();
	new_post = 0;
	for( long i = 0; i < new_blocks.size(); i ++ )
	{
		if( i == block_num )
		{
			temp_blocks.push_back( block_1 );
			temp_blocks.push_back( block_2 );
			
			new_post += block_1.get_current_post() + change_point_prior;
			new_post += block_2.get_current_post() + change_point_prior;
			
			i ++;
		}else
		{
			temp_blocks.push_back( new_blocks[i] );
			new_post += blocks[i].get_current_post() + change_point_prior;
		}
		
	}
	
	new_blocks = temp_blocks;
	return 0;
}

int BCCPM::metro_hast_run()
{
	long i, j;
	
	samples.clear();
	traces.clear();
	
	samples.resize( num_iter_1 );
	traces.resize( num_iter_1 );

	BCCPMBlock * block_ptr;
	long length = 0;
	blocks.clear();
	for( i = 0; i< init_block_lengths.size(); i ++ )
	{
		block_ptr = new BCCPMBlock(num_pos, hyper_par_group, hyper_par_single);
		block_ptr->set_data(length, init_block_lengths[i], data);
		length += init_block_lengths[i];
		blocks.push_back(*block_ptr);
	}
		
	time_t t;
	t = time( NULL );
	gsl_rng_default_seed = t;
	gsl_rng * rng = gsl_rng_alloc( gsl_rng_taus );
	double  temp_prior=change_point_prior;

	int propose;
	vector< BCCPMBlock > new_blocks;
	double old_post = 0;
	double new_post = 0;

	for( i = 0; i < blocks.size(); i ++ )
	{
		blocks[i].calc_post();
//		blocks[i].metro_hast_run( num_iter_2, 10 );
		old_post += blocks[i].get_current_post() + change_point_prior;
	}
	samples[0] = blocks;
	traces[0] = old_post;
	
	
	for( i = 1; i < num_iter_1; i ++ )
	{
		cout << i << endl;
		
		new_blocks = blocks;	
		propose = gsl_rng_uniform_int( rng, 3 );

		switch( propose )
		{
			case 0:
				propose_divide( new_post, new_blocks, rng );
				break;
			case 1:
				propose_combine( new_post, new_blocks, rng );
				break;
			case 2:
				propose_shift( new_post, new_blocks, rng );
				break;
			default:
				propose_shift( new_post, new_blocks, rng );
				break;
		}
		if( new_post == GSL_NEGINF )
		{
			i --;
			continue;
		}
		//cout << old_post << '\t' << new_post << '\t' << blocks.size() << '\t';

		if( gsl_rng_uniform( rng ) < exp( new_post - old_post ) )
		{
			blocks = new_blocks;
			old_post= new_post;
		//	cout << "Accepted\t";
		}

		old_post = 0;

		for( j = 0; j < blocks.size(); j ++ )
		{
			//blocks[j].initialize();
            
			//debug
			//blocks[j].metro_hast_run( num_iter_2, 10 );
			blocks[j].calc_post();
		//	blocks[j].metro_hast_run( 10, 1 );
			
			old_post += blocks[j].get_current_post() + change_point_prior;
		}
		//cout << old_post << '\t' << blocks.size() << endl;
		samples[i] = blocks;
		traces[i] = old_post;
		
		
	}
	convert_to_block_labels();
	CaluPost_CP();
	gsl_rng_free( rng );
	
	return 0;
}



int BCCPM::load_data( long num_sample, long num_pos, string data_file, string par_file, string init_block_file)
{
	this -> num_sample = num_sample;
	this -> num_pos = num_pos;
	
	if( num_pos < 2 )
	{
		cout << "Data must contain at least 2 columns." << endl;
		abort();
	}
	
	
	data = gsl_matrix_calloc( num_sample, num_pos );
	FILE * data_stream = fopen( data_file.c_str(), "r" );
	if( NULL == data_stream )
	{
		cout << "File doesn't exist." << endl;
		abort();
	}
	
	if( GSL_EFAILED == gsl_matrix_fscanf( data_stream, data ) )
	{
		cout << "Wrong data file." << endl;
		abort();
	}
	
	fclose( data_stream );
	
	ifstream init_block_stream;
	init_block_lengths.clear();
	init_block_stream.open(init_block_file.c_str());
	string length;
	if( init_block_stream.good())
	{
		while(init_block_stream >> length)
		{
			init_block_lengths.push_back(atoi(length.c_str()));
		}
	}
	init_block_stream.close();

	ifstream par_stream;
	par_stream.open( par_file.c_str() );
	
	if( !par_stream.good() )
	{
		cout << "File doesn't exist." << endl;
		return 1;
	}
	vector< double > pars;
	double temp_par;
	string par_name;
	par_stream >> par_name;
	while( par_stream >> temp_par )
	{
		pars.push_back( temp_par );
		par_stream >> par_name;
	}
	if( pars.size() != 9 )
	{
		cout << "Wrong parameter file." << endl;
		abort();
	}
	par_stream.close();
	hyper_par_single.mu = pars[0];
	hyper_par_single.nu = pars[1];
	hyper_par_single.kappa = pars[2];
	hyper_par_single.lamda = pars[3];
	
	hyper_par_group.mu = pars[4];
	hyper_par_group.nu = pars[5];
	hyper_par_group.kappa = pars[6];
	hyper_par_group.lamda_diag = pars[7];
	hyper_par_group.lamda_off_diag = pars[8];
	
	return 0;
	
}

int BCCPM::convert_to_block_labels()
{
	long num_iter = samples.size();
	block_labels.resize( num_iter );
	long i, j;
	for( i = 0; i < num_iter; i ++ )
	{
		block_labels[i].resize( num_sample );
		for( j = 0; j < num_sample; j ++ )
		{
			block_labels[i][j] = 0;
		}
		for( j = 0; j < samples[i].size(); j ++ )
		{
			block_labels[i][samples[i][j].get_start()] = 1;
		}
	}
	return 0;
}



int BCCPM::output_results( string trace_file, string block_label_file, string post_file, int thin )
{
	set_thin( thin );

	int i, j;
	
	ofstream trace_stream;
	trace_stream.open( trace_file.c_str() );
	
	for( i = 0; i < thinned_size; i ++ )
	{
		trace_stream << traces[i*thin] << endl;
	}
	trace_stream.close();
	
	ofstream block_label_stream;
	block_label_stream.open( block_label_file.c_str() );
	for( i = 0; i < thinned_size; i ++ )
	{
		for( j = 0; j < block_labels[i*thin].size(); j ++ )
		{
			block_label_stream << block_labels[i*thin][j] << '\t';
		}
		block_label_stream << endl;
	}
	block_label_stream.close();

	ofstream post_stream;
	post_stream.open( post_file.c_str() );
	
	for( i = 0; i < posts.size(); i ++ )
	{
		post_stream << posts[i] << endl;
	}
	post_stream.close();
	
	return 0;
}


int BCCPM::CaluPost_CP()
{
	int i, j;
	double post;
	posts.clear();
	posts.assign(num_sample,0);
	for (j=0;j<num_sample;j++){
		post=0;
		for( i = burn_in; i < num_iter_1; i ++ )
		{
			post+=block_labels[i][j];
		}
		posts[j]=post/(num_iter_1-burn_in);
	}
	
	return 0;
}


double BCCPM::validate_blocks( vector< int > & block_labels, long num_iter )
{
	BCCPMBlock temp_block( num_pos, hyper_par_group, hyper_par_single );
	long i;
	long start = 0;
	long num_sample = 0;
	for( i = 0; i < block_labels.size(); i ++ )
	{
		if( block_labels[i] == 1 )
		{
			if( num_sample )
			{
				temp_block.set_data( start, num_sample, data );
				blocks.push_back( temp_block );
			}
			start = i;
			num_sample = 0;
		}
		num_sample ++;
	}
	if( num_sample )
	{
		temp_block.set_data( start, num_sample, data );
		blocks.push_back( temp_block );
	}
	
	double post = 0;
	
	for( i = 0; i < blocks.size(); i ++ )
	{
//		blocks[i].initialize();
//		blocks[i].metro_hast_run( num_iter, 10 );
		blocks[i].calc_post();
		post += blocks[i].get_current_post() + change_point_prior;
	}
	return post;
}

int BCCPM::set_num_iter( long num_iter_1)
{
	this->num_iter_1 = num_iter_1;
	return 0;
}

int BCCPM::set_thin( int thin )
{
	this->thin = thin;
	this->thinned_size = num_iter_1/thin;
	
	return 0;
}

int BCCPM::set_burn_in( long burn_in )
{
	this->burn_in = burn_in;
	
	return 0;
}

int BCCPM::set_change_point_prior( double change_point_prior )
{
	this->change_point_prior = change_point_prior;
	return 0;
}
