/*
 *  BCCPMBlock.cpp
 *  
 *  Created by Zhichao Lian on Apr/10/2013.
 *  Copyright 2013 Yale University. All rights reserved.
 *
 */

#include "BCCPMBlock.h"

BCCPMBlock::BCCPMBlock( int num_pos, HyperParGroup & hyper_par_group, HyperParSingle & hyper_par_single )
{
	this->num_pos = num_pos;
	this->hyper_par_group = hyper_par_group;
	this->hyper_par_single = hyper_par_single;
	//initial_grouping.resize( num_pos );
	
	num_sample = 0;
}

BCCPMBlock::~BCCPMBlock()
{}


int BCCPMBlock::set_data( long start, long num_sample, gsl_matrix * source )
{
	this->block_data = gsl_matrix_submatrix( source, start, 0, num_sample, num_pos );
	this->num_sample = num_sample;
	this->start = start;
	
	return 0;
}

int BCCPMBlock::calc_post()
{
	int i,j;
	gsl_matrix * group_data = NULL;
	gsl_vector * single_data = NULL;
	group_data = gsl_matrix_calloc( num_sample, num_pos);
	for( i = 0; i < num_sample; i ++ )
	{
		for( j = 0; j < num_pos; j ++ )
		{
				gsl_matrix_set( group_data, i, j, gsl_matrix_get( &(block_data.matrix), i, j ) );
		}
	}

	if( group_data == NULL )
	{
		current_post = 0;
		return 0;
	}
	if( group_data->size2 == 1 )
	{
		single_data = gsl_vector_calloc( num_sample );
		gsl_matrix_get_col( single_data, group_data, 0 );
		gsl_matrix_free( group_data );
		
		calc_single_lik( single_data, current_post );
		gsl_vector_free( single_data );
	}else
	{
		calc_group_lik( group_data, current_post);
		gsl_matrix_free( group_data );
	}	
	return 0;
    
}

int BCCPMBlock::calc_group_lik( gsl_matrix * group_data, double & likelihood )
{
	int i, j, k;
	double temp;
	
	int c = group_data -> size2;
	long m = group_data -> size1;
	
	gsl_vector * mu;
	double kappa, kappa_m;
	double nu, nu_m;
	gsl_matrix * lamda;
	gsl_matrix * lamda_m;
	gsl_matrix * s;
	gsl_matrix * s_m;
	lamda = gsl_matrix_calloc( c, c );
	lamda_m = gsl_matrix_calloc( c, c );
	s = gsl_matrix_calloc( c, c );
	s_m = gsl_matrix_calloc( c, c );
	mu = gsl_vector_calloc(c);
	
	kappa = hyper_par_group.kappa;
	////////
	nu = hyper_par_group.nu*c;
	////////
	kappa_m = kappa + m;
	nu_m = nu + m;
	
	vector< double > mean;
	mean.resize( c );
	for( i = 0; i < c; i ++ )
	{
		temp = 0;
		for( j = 0; j < m; j ++ )
		{
			temp += gsl_matrix_get( group_data, j, i );
		}
		mean[i] = temp/m;
	}
	
	gsl_vector_set_all( mu, hyper_par_group.mu );
	for( i = 0; i < c; i ++ )
	{
		for( j = 0; j < c; j ++ )
		{
			if( i == j)
			{
				gsl_matrix_set( lamda, i, j, hyper_par_group.lamda_diag );
				gsl_matrix_set( lamda_m, i, j, hyper_par_group.lamda_diag );
			}
			else
			{
				gsl_matrix_set( lamda, i, j, hyper_par_group.lamda_off_diag );
				gsl_matrix_set( lamda_m, i, j, hyper_par_group.lamda_off_diag );
			}
			temp = (mean[i] - gsl_vector_get(mu, i))*(mean[j] - gsl_vector_get(mu, j))*kappa*m/kappa_m;
			gsl_matrix_set( s_m, i, j, temp);
			for( k = 0; k < m; k ++ )
			{
				temp = gsl_matrix_get(s, i, j);
				temp += (gsl_matrix_get(group_data, k, i) - mean[i])*(gsl_matrix_get(group_data, k, j) - mean[j]);
				gsl_matrix_set( s, i, j, temp );
			}
		}
	}
	
	gsl_matrix_add( lamda_m, s );
	gsl_matrix_add( lamda_m, s_m );
	
	likelihood = -m*c*0.5*log(2*PI)+c*0.5*log(kappa/kappa_m);
	for( i = 0; i < c; i ++ )
	{
		likelihood += gsl_sf_lngamma(nu_m*0.5+(-i)*0.5) - gsl_sf_lngamma(nu*0.5+(-i)*0.5);
	}
	
	gsl_permutation * p = gsl_permutation_alloc( c );
	int signum;
	gsl_linalg_LU_decomp(lamda, p, &signum);
	gsl_linalg_LU_decomp(lamda_m, p, &signum);
	likelihood += nu*0.5*gsl_linalg_LU_lndet(lamda) - nu_m*0.5*gsl_linalg_LU_lndet(lamda_m)+m*c*0.5*log(2);
	
	
	gsl_matrix_free(lamda);
	gsl_matrix_free(lamda_m);
	gsl_matrix_free(s);
	gsl_matrix_free(s_m);
	gsl_vector_free(mu);
	gsl_permutation_free(p);
	
	return 0;
}

int BCCPMBlock::calc_single_lik( gsl_vector * single_data, double & likelihood )
{
	double mu = hyper_par_single.mu;
	double nu = hyper_par_single.nu;
	double kappa = hyper_par_single.kappa;
	double lamda = hyper_par_single.lamda;
	int i;
	
	long m = single_data -> size;
	double nu_m = nu + m;
	double kappa_m = kappa + m;
	
	double mean = 0;
	double var = 0;
	for( i = 0; i < m; i ++ )
	{
		mean += gsl_vector_get( single_data, i);
	}
	mean /= m;
	for( i = 0; i < m; i ++ )
	{
		var += pow( gsl_vector_get( single_data, i) - mean, 2 );
	}
	var /= m-1;
	
	double lamda_m = lamda + (m-1)*var + (kappa*m)*(mean-mu)*(mean-mu)/kappa_m;
	likelihood = -0.5*m*log(2*PI) + 0.5*log(kappa/kappa_m) + gsl_sf_lngamma(nu_m*0.5) - gsl_sf_lngamma(nu*0.5) + nu*0.5*log(lamda*0.5) - nu_m*0.5*log(lamda_m*0.5);
	
	return 0;
}

long BCCPMBlock::get_num_sample()
{
	return num_sample;
}

long BCCPMBlock::get_start()
{
	return start;
}

double BCCPMBlock::get_current_post()
{
	return current_post;
}
