/*
 *  DBVPM.h
 *
 *  Created by Cong Li on 7/8/11.
 *  Revised by Zhichao Lian.
 *  Copyright 2011 Yale University. All rights reserved.
 *
 */


#ifndef _DBVPM_H_
#define _DBVPM_H_

#include "DBVPMBlock.h"


class DBVPM
{
	HyperParSingle hyper_par_single;
	HyperParGroup hyper_par_group;
	
	int thin;
	long thinned_size;
	
	long num_iter_1;
	long num_iter_2;
	
	gsl_matrix * data;
	long num_sample;
	int num_pos;
	double change_point_prior;
	int burn_in;
	
	vector< int > init_block_lengths;
	vector< vector< DBVPMBlock > > samples;
	vector< double > traces;
	vector< vector< int > > block_labels;
	vector< DBVPMBlock > blocks;
	vector <double> posts;
	
	int propose_divide( double & new_post, vector< DBVPMBlock> & new_blocks, gsl_rng * rng );
	int propose_combine( double & new_post, vector< DBVPMBlock > & new_blocks, gsl_rng * rng );
	int propose_shift( double & new_post, vector< DBVPMBlock > & new_blocks, gsl_rng * rng );
	int CaluPost_CP();
	int convert_to_block_labels();

public:
	int set_change_point_prior( double change_point_prior );
	int set_thin( int thin );
	int set_burn_in( int burn_in );
	int load_data( long num_sample, long num_pos, string data_file, string par_file, string init_block_file );
	int output_results( string trace_file, string block_label_file, string change_point_post_file, int thin );
	int metro_hast_run_init();
	double validate_blocks( vector< int > & block_labels, long nIter );
	int get_num_pos();
	long get_num_sample();
	int set_num_iter( long num_iter_1, long num_iter_2 );
	
	DBVPM();
	~DBVPM();
	
};


#endif
