/*
 *  DBVPMBlock.h
 *
 *  Created by Cong Li on 7/8/11.
 *  Revised by Zhichao Lian 
 *  Copyright 2011 Yale University. All rights reserved.
 *
 */

#ifndef _DBVPMBLOCK_H_
#define _DBVPMBLOCK_H_

#include "CommonHead.h"

using namespace std;


class DBVPMBlock
{
	HyperParSingle hyper_par_single;
	HyperParGroup hyper_par_group;
	
	gsl_matrix_view block_data;
	long start;
	long num_sample;
	long num_pos;
	
	vector< int > initial_grouping;
	int initial_struct;
	double initial_post;
	
	vector< int > grouping_sample;
	int str_sample;
	double current_post;
	
	int is_valid_grouping( vector< int > & grouping );
	int get_group_data( vector< int > & grouping, vector< int > & group_mbr, gsl_matrix * & group_data );
	int calc_lik( vector< int > & grouping, vector< int > & group_mbr, double & likelihood );
	int calc_group_lik( gsl_matrix * group_data, double & likelihood );
	int calc_single_lik( gsl_vector * single_data, double & likelihood );
	
	int propose_mutate( vector< int > & grouping, int str, double & new_post, gsl_rng * rng );
	int propose_swap( vector< int > & grouping, int str, double & new_post, gsl_rng * rng );
	int propose_str( vector< int > & grouping, int & str, double & new_post, gsl_rng * rng );
	
public:
	int set_data( long start, long num_sample, gsl_matrix * source );
	double get_current_post();
	int get_last_grouping( vector< int > & grouping );
	int get_last_struct( int & str );
	
	int initialize();
	int calc_post( vector< int > & grouping, int str, double & posterior );
	int metro_hast_run( long n_iter, double temperature );
	
	long get_num_sample();
	long get_start();
	
	DBVPMBlock( int num_pos, HyperParGroup & hyper_par_group, HyperParSingle & hyper_par_single );
	~DBVPMBlock();
};

#endif