#include "BCCPM.h"
#include "DBVPM.h"

int main (int argc, char * const argv[]) {
//int main(){
	
	if( argc != 13 )
	{
		cout << "Usage:"<<endl;   
		cout << "DetectCP(.exe) [method] [data file] [number of samples] [number of variables] [number of high-level iterations][number of low-level iterations] [block label file] [trace file] [post file] [burn-in] [change point prior] [init file]" << endl;
		cout << "[method]:" <<endl;
		cout << "         1: DBVPM" <<endl;
		cout << "         2: BCCPM" <<endl;
		cout << "[data file]:"<<endl;
		cout << "         input file, must be a n*m matrix where n is [number of samples] and m is [number of variables] (ROI number).  "<<endl;
		cout << "[number of samples]:"<<endl;
		cout << "                   the number of samples, also the number of line in the input file." <<endl;
		cout << "[number of variables]:"<<endl;
		cout << "                   the number of variables, also the number of columns in the input file." <<endl;
		cout << "[number of high-level iterations]: "<<endl;
		cout << "                         suggested to be greater than 1000 at least."<<endl;
		cout << "[number of low-level iterations]: "<<endl;
		cout << "                         suggested to be greater than 100 at least."<<endl;
		cout <<" [block label file]:"<<endl;
		cout <<"                   output file, and it is used to calculate posterior probabilities of each time point being a change point"<<endl;
		cout <<" [trace file]:"<<endl;
		cout <<"              output file, it is used to confirm the MCMC procedure is converged."<<endl;
		cout <<" [post file]:"<<endl;
		cout <<"             output file, it contains the posterior probabilities of each time point being a change point."<<endl;
		cout <<" [burn-in]:"<<endl;
		cout <<"          a integer value, must be smaller than the number of high-level iteration."<<endl;
		cout <<" [change point prior]:"<<endl;
		cout <<"          a doule value, it is normally set as 0. Positive value will encourage to detect more change points. While less change points are desired, negative value is preferred."<<endl;
		cout <<" [init file]:" << endl;
		cout <<"            it contains the prior information on the length of each block. If we do not know anything, it only contains a value (the number of samples). Otherwise, the init file should contain several lines each of which is the length of a corresponding block. "<<endl;
		cout <<"            Please note that the sum of the values in  [init file] must equal to the number of samples, otherwise the code will abort."<<endl;
		return 0;
	}
	
	string data_file;
	string hyper_par_file;
	string block_label_file;
	string trace_file;
	string post_file;
	string init_block_file;

	
	data_file = argv[2];
	hyper_par_file = "hyper_par.txt";
	block_label_file = argv[7];
	trace_file = argv[8];
	post_file =argv[9];
	init_block_file = argv[12];


	int method= atoi(argv[1]);
	int thin = 1;
	long burn_in = atoi(argv[10]);

	long num_iter_1 = atol(argv[5]);
	long num_iter_2 = atol(argv[6]);
	
	long num_sample = atol(argv[3]);
	long num_pos = atol(argv[4]);
	
	double change_point_prior = atof(argv[11]);

    
/*
	//for debug
	data_file = "newdata_Figure2_1.txt";
	hyper_par_file = "hyper_par.txt";
	block_label_file = "block_label.txt";
	trace_file = "trace.txt";
	post_file = "post.txt";
	init_block_file= "init.txt";
	
	int method=2;
	int thin = 1;
	long burn_in=800;
	long num_iter_1 = 1000;
	long num_iter_2 = 100;
	long num_sample = 200;
	long num_pos = 10;
	
	float change_point_prior = 0;
*/

    switch (method){ 
		case 1: {
			DBVPM DBVPM_rms;
			DBVPM_rms.set_thin(thin );
			DBVPM_rms.set_burn_in( burn_in );
			DBVPM_rms.set_change_point_prior( change_point_prior );
			DBVPM_rms.set_num_iter( num_iter_1, num_iter_2 );
			DBVPM_rms.load_data( num_sample, num_pos, data_file, hyper_par_file, init_block_file);

			DBVPM_rms.metro_hast_run_init();
			DBVPM_rms.output_results( trace_file, block_label_file, post_file, thin );
			break;
		}
		case 2: {

			BCCPM BCCPM_rms;
			BCCPM_rms.set_burn_in( burn_in );
			BCCPM_rms.set_change_point_prior( change_point_prior );
			BCCPM_rms.set_num_iter( num_iter_1);
			BCCPM_rms.load_data( num_sample, num_pos, data_file, hyper_par_file, init_block_file);
			BCCPM_rms.metro_hast_run();
			BCCPM_rms.output_results( trace_file, block_label_file, post_file,thin );
			break;
		}default: {
			BCCPM rms;
			rms.set_change_point_prior( change_point_prior );
			rms.set_num_iter( num_iter_1);
			rms.load_data( num_sample, num_pos, data_file, hyper_par_file, init_block_file);
			rms.metro_hast_run();
			rms.output_results( trace_file, block_label_file, post_file,thin );
			break;
		}

	}
	return 0;
}
